@@ -4,6 +4,7 @@ import copy
 import json
 import os
 import sys
+import typing
 from enum import Enum
 from pathlib import Path
 
@@ -12,7 +13,7 @@ import typer
 import uvicorn
 from cookiecutter.main import cookiecutter
 
-from unionml.remote import VersionFetchError, get_app_version, get_model, get_model_execution
+from unionml.remote import VersionFetchError, get_app_version, get_model, get_model_execution, get_prediction_execution
 
 sys.path.append(os.curdir)
 
@@ -27,9 +28,14 @@ class AppTemplate(str, Enum):
     basic = "basic"
     basic_aws_lambda = "basic-aws-lambda"
     basic_aws_lambda_s3 = "basic-aws-lambda-s3"
+    basic_bentoml = "basic-bentoml"
     quickdraw = "quickdraw"
 
 
+class PredictionOutputFormats(str, Enum):
+    json = "json"
+
+
 @app.command()
 def init(
     app_name: str,
@@ -64,12 +70,17 @@ def deploy(
         "--patch",
         help="Bypass Docker build process and update the UnionML app source code using the latest available image.",
     ),
+    schedule: bool = typer.Option(
+        True,
+        "--schedule/--no-schedule",
+        help="Indicates whether or not to deploy the training and prediction schedules.",
+    ),
 ):
     """Deploy model to a Flyte backend."""
     typer.echo(f"[unionml] deploying {app}")
     model = get_model(app)
     try:
-        model.remote_deploy(allow_uncommitted=allow_uncommitted, patch=patch)
+        model.remote_deploy(allow_uncommitted=allow_uncommitted, patch=patch, schedule=schedule)
     except VersionFetchError as e:
         typer.echo(f"[unionml] failed to get app version: {e}", err=True)
         typer.echo(
@@ -82,6 +93,38 @@ def deploy(
         raise typer.Exit(code=1)
 
 
+@app.command()
+def activate_schedules(
+    app: str,
+    app_version: str = typer.Option(None, "--app-version", "-v", help="App version"),
+    schedule_names: typing.List[str] = typer.Option(
+        None,
+        "--name",
+        help="Name of the schedule to activate. This option can be specified multiple times.",
+    ),
+):
+    """Activate training and prediction schedules specified in your UnionML app."""
+    typer.echo(f"[unionml] activating schedules {schedule_names} for {app}")
+    model = get_model(app)
+    model.remote_activate_schedules(app_version=app_version, schedule_names=schedule_names)
+
+
+@app.command()
+def deactivate_schedules(
+    app: str,
+    app_version: str = typer.Option(None, "--app-version", "-v", help="app version"),
+    schedule_names: typing.List[str] = typer.Option(
+        None,
+        "--name",
+        help="Name of the schedule to deactivate. This option can be specified multiple times.",
+    ),
+):
+    """Deactivate training and prediction schedules specified in your UnionML app."""
+    typer.echo(f"[unionml] deactivating schedules {schedule_names} for {app}")
+    model = get_model(app)
+    model.remote_activate_schedules(app_version=app_version, schedule_names=schedule_names)
+
+
 @app.command()
 def train(
     app: str,
@@ -136,7 +179,6 @@ def list_model_versions(
     ),
 ):
     r"""List all model versions."""
-
     model = get_model(app)
     app_version = app_version or get_app_version(allow_uncommitted=True)
     typer.echo(f"[unionml] app: {app} - listing model versions for app version={app_version}")
@@ -144,6 +186,52 @@ def list_model_versions(
         typer.echo(f"- {model_version}")
 
 
+@app.command("list-prediction-ids")
+def list_prediction_ids(
+    app: str,
+    app_version: str = typer.Option(None, "--app-version", "-v", help="app version"),
+    limit: int = typer.Option(
+        10, "--limit", help="Maximum number of model versions to list, sorted in descending order of time of execution."
+    ),
+):
+    r"""List all batch prediction identifiers."""
+    model = get_model(app)
+    app_version = app_version or get_app_version(allow_uncommitted=True)
+    typer.echo(f"[unionml] app: {app} - listing prediction ids for app version={app_version}")
+    for prediction_id in model.remote_list_prediction_ids(app_version, limit):
+        typer.echo(f"- {prediction_id}")
+
+
+@app.command()
+def list_scheduled_training_runs(
+    app: str,
+    schedule_name,
+    app_version: str = typer.Option(None, "--app-version", "-v", help="app version"),
+    limit: int = typer.Option(5, "--limit", help="number of "),
+):
+    r"""List scheduled training runs by schedule name."""
+    model = get_model(app)
+    app_version = app_version or get_app_version(allow_uncommitted=True)
+    typer.echo(f"[unionml] app: {app} - listing scheduled training runs for schedule '{schedule_name}'")
+    for run in model.remote_list_scheduled_training_runs(schedule_name, app_version=app_version, limit=limit):
+        typer.echo(f"- {run.id.name}")
+
+
+@app.command()
+def list_scheduled_prediction_runs(
+    app: str,
+    schedule_name,
+    app_version: str = typer.Option(None, "--app-version", "-v", help="app version"),
+    limit: int = typer.Option(5, "--limit", help="number of "),
+):
+    r"""List scheduled prediction runs by schedule name."""
+    model = get_model(app)
+    app_version = app_version or get_app_version(allow_uncommitted=True)
+    typer.echo(f"[unionml] app: {app} - listing scheduled training runs for schedule '{schedule_name}'")
+    for run in model.remote_list_scheduled_training_runs(schedule_name, app_version=app_version, limit=limit):
+        typer.echo(f"- {run.id.name}")
+
+
 @app.command("fetch-model")
 def fetch_model(
     app: str,
@@ -164,6 +252,32 @@ def fetch_model(
     typer.echo(f"[unionml] app: {app} - saving model version {execution.id.name} to {output_file}")
 
 
+@app.command("fetch-predictions")
+def fetch_predictions(
+    app: str,
+    app_version: str = typer.Option(None, "--app-version", "-v", help="app version"),
+    prediction_id: str = typer.Option("latest", "--prediction-id", "-m", help="prediction id"),
+    output_file: str = typer.Option(None, "--output-file", "-o", help="output file path"),
+    output_format: str = typer.Option(
+        PredictionOutputFormats.json,
+        "--output-format",
+        "-f",
+        help="Output format of the file. Currently only json-serializable prediction outputs are supported.",
+    ),
+):
+    r"""Fetch a model object from the remote backend."""
+    model = get_model(app)
+    app_version = app_version or get_app_version(allow_uncommitted=True)
+    execution = get_prediction_execution(model, app_version, prediction_id=prediction_id)
+    predictions = model.remote_fetch_predictions(execution)
+    if output_format == "json":
+        with open(output_file) as f:
+            json.dump(predictions, f)
+    else:
+        raise ValueError(f"output_format '{output_format}' not recognized.")
+    typer.echo(f"[unionml] app: {app} - saving predictions {execution.id.name} to {output_file}")
+
+
 @app.callback()
 def callback():
     r"""unionml command-line tool."""
@@ -175,8 +289,8 @@ def serve_command():
     fn.name = "serve"
     fn.short_help = "Serve an unionml model."
     fn.help = (
-        "Serve an unionml model using uvicorn. This command uses the main uvicorn entrypoint with an additional "
-        "``--model-path`` argument.\n\nFor more information see: https://www.uvicorn.org/#command-line-options"
+        "Serve an unionml model using uvicorn. This command uses the main uvicorn entrypoint with an additional"
+        " ``--model-path`` argument.\n\nFor more information see: https://www.uvicorn.org/#command-line-options"
     )
 
     option = click.Option(param_decls=["--model-path"], default=None, help="model path to use for serving", type=Path)
